@@ -3,7 +3,8 @@ package Moose::Meta::Method;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION   = '0.51';
+our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Class::MOP::Method';
 
@@ -33,15 +34,13 @@ to cpan-RT.
 
 Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
-Yuval Kogman E<lt>nothingmuch@woobling.comE<gt>
-
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006 by Infinity Interactive, Inc.
+Copyright 2006-2008 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-=cut
\ No newline at end of file
+=cut
