@@ -5,11 +5,10 @@ use strict;
 use warnings;
 
 use Carp         'confess';
-use Scalar::Util 'blessed', 'reftype';
-use B            'svref_2object';
+use Scalar::Util 'blessed';
 use Sub::Exporter;
 
-our $VERSION   = '0.16';
+our $VERSION   = '0.51';
 our $AUTHORITY = 'cpan:STEVAN';
 
 ## --------------------------------------------------------
@@ -21,13 +20,21 @@ our $AUTHORITY = 'cpan:STEVAN';
 
 # creation and location
 sub find_type_constraint                 ($);
+sub register_type_constraint             ($);
 sub find_or_create_type_constraint       ($;$);
+sub find_or_parse_type_constraint        ($);
+sub find_or_create_isa_type_constraint   ($);
+sub find_or_create_does_type_constraint  ($);
 sub create_type_constraint_union         (@);
 sub create_parameterized_type_constraint ($);
+sub create_class_type_constraint         ($;$);
+sub create_role_type_constraint          ($;$);
+sub create_enum_type_constraint          ($$);
 
 # dah sugah!
 sub type        ($$;$$);
 sub subtype     ($$;$$$);
+sub class_type  ($;$);
 sub coerce      ($@);
 sub as          ($);
 sub from        ($);
@@ -46,15 +53,21 @@ sub _install_type_coercions ($$);
 use Moose::Meta::TypeConstraint;
 use Moose::Meta::TypeConstraint::Union;
 use Moose::Meta::TypeConstraint::Parameterized;
+use Moose::Meta::TypeConstraint::Parameterizable;
+use Moose::Meta::TypeConstraint::Class;
+use Moose::Meta::TypeConstraint::Role;
+use Moose::Meta::TypeConstraint::Enum;
 use Moose::Meta::TypeCoercion;
 use Moose::Meta::TypeCoercion::Union;
 use Moose::Meta::TypeConstraint::Registry;
+use Moose::Util::TypeConstraints::OptimizedConstraints;
 
 my @exports = qw/
-    type subtype as where message optimize_as
+    type subtype class_type role_type as where message optimize_as
     coerce from via
     enum
     find_type_constraint
+    register_type_constraint
 /;
 
 Sub::Exporter::setup_exporter({
@@ -72,7 +85,7 @@ sub unimport {
             my $keyword = \&{$class . '::' . $name};
 
             # make sure it is from Moose
-            my $pkg_name = eval { svref_2object($keyword)->GV->STASH->NAME };
+            my ($pkg_name) = Class::MOP::get_code_info($keyword);
             next if $@;
             next if $pkg_name ne 'Moose::Util::TypeConstraints';
 
@@ -94,8 +107,8 @@ sub export_type_constraints_as_functions {
     my $pkg = caller();
     no strict 'refs';
     foreach my $constraint (keys %{$REGISTRY->type_constraints}) {
-        *{"${pkg}::${constraint}"} = $REGISTRY->get_type_constraint($constraint)
-                                              ->_compiled_type_constraint;
+        my $tc = $REGISTRY->get_type_constraint($constraint)->_compiled_type_constraint;
+        *{"${pkg}::${constraint}"} = sub { $tc->($_[0]) ? 1 : undef }; # the undef is for compat
     }
 }
 
@@ -139,37 +152,60 @@ sub create_parameterized_type_constraint ($) {
     return Moose::Meta::TypeConstraint::Parameterized->new(
         name           => $type_constraint_name,
         parent         => $REGISTRY->get_type_constraint($base_type),
-        type_parameter => find_or_create_type_constraint(
-            $type_parameter => {
-                parent     => $REGISTRY->get_type_constraint('Object'),
-                constraint => sub { $_[0]->isa($type_parameter) }
-            }
-        ),
+        type_parameter => find_or_create_isa_type_constraint($type_parameter),
     );
 }
 
+#should we also support optimized checks?
+sub create_class_type_constraint ($;$) {
+    my ( $class, $options ) = @_;
+
+    # too early for this check
+    #find_type_constraint("ClassName")->check($class)
+    #    || confess "Can't create a class type constraint because '$class' is not a class name";
+
+    my %options = (
+        class => $class,
+        name  => $class,
+        %{ $options || {} },
+    );
+
+    $options{name} ||= "__ANON__";
+
+    Moose::Meta::TypeConstraint::Class->new( %options );
+}
+
+sub create_role_type_constraint ($;$) {
+    my ( $role, $options ) = @_;
+
+    # too early for this check
+    #find_type_constraint("ClassName")->check($class)
+    #    || confess "Can't create a class type constraint because '$class' is not a class name";
+
+    my %options = (
+        role => $role,
+        name => $role,
+        %{ $options || {} },
+    );
+
+    $options{name} ||= "__ANON__";
+
+    Moose::Meta::TypeConstraint::Role->new( %options );
+}
+
+
 sub find_or_create_type_constraint ($;$) {
-    my ($type_constraint_name, $options_for_anon_type) = @_;
+    my ( $type_constraint_name, $options_for_anon_type ) = @_;
 
-    return $REGISTRY->get_type_constraint($type_constraint_name)
-        if $REGISTRY->has_type_constraint($type_constraint_name);
-
-    my $constraint;
-
-    if (_detect_type_constraint_union($type_constraint_name)) {
-        $constraint = create_type_constraint_union($type_constraint_name);
+    if ( my $constraint = find_or_parse_type_constraint($type_constraint_name) ) {
+        return $constraint;
     }
-    elsif (_detect_parameterized_type_constraint($type_constraint_name)) {
-        $constraint = create_parameterized_type_constraint($type_constraint_name);
-    }
-    else {
+    elsif ( defined $options_for_anon_type ) {
         # NOTE:
-        # if there is no $options_for_anon_type 
-        # specified, then we assume they don't 
+        # if there is no $options_for_anon_type
+        # specified, then we assume they don't
         # want to create one, and return nothing.
-        return unless defined $options_for_anon_type;        
 
-        # NOTE:
         # otherwise assume that we should create
         # an ANON type with the $options_for_anon_type
         # options which can be passed in. It should
@@ -182,6 +218,36 @@ sub find_or_create_type_constraint ($;$) {
         );
     }
 
+    return;
+}
+
+sub find_or_create_isa_type_constraint ($) {
+    my $type_constraint_name = shift;
+    find_or_parse_type_constraint($type_constraint_name) || create_class_type_constraint($type_constraint_name)
+}
+
+sub find_or_create_does_type_constraint ($) {
+    my $type_constraint_name = shift;
+    find_or_parse_type_constraint($type_constraint_name) || create_role_type_constraint($type_constraint_name)
+}
+
+sub find_or_parse_type_constraint ($) {
+    my $type_constraint_name = shift;
+
+    return $REGISTRY->get_type_constraint($type_constraint_name)
+        if $REGISTRY->has_type_constraint($type_constraint_name);
+
+    my $constraint;
+
+    if (_detect_type_constraint_union($type_constraint_name)) {
+        $constraint = create_type_constraint_union($type_constraint_name);
+    }
+    elsif (_detect_parameterized_type_constraint($type_constraint_name)) {
+        $constraint = create_parameterized_type_constraint($type_constraint_name);
+    } else {
+        return;
+    }
+
     $REGISTRY->add_type_constraint($constraint);
     return $constraint;
 }
@@ -190,7 +256,22 @@ sub find_or_create_type_constraint ($;$) {
 ## exported functions ...
 ## --------------------------------------------------------
 
-sub find_type_constraint ($) { $REGISTRY->get_type_constraint(@_) }
+sub find_type_constraint ($) {
+    my $type = shift;
+
+    if ( blessed $type and $type->isa("Moose::Meta::TypeConstraint") ) {
+        return $type;
+    } else {
+        return $REGISTRY->get_type_constraint($type);
+    }
+}
+
+sub register_type_constraint ($) {
+    my $constraint = shift;
+    confess "can't register an unnamed type constraint" unless defined $constraint->name;
+    $REGISTRY->add_type_constraint($constraint);
+    return $constraint;
+}
 
 # type constructors
 
@@ -209,10 +290,28 @@ sub subtype ($$;$$$) {
     #   subtype(MyNumbers => as Num); # now MyNumbers is the same as Num
     # ... yeah I know it's ugly code
     # - SL
-    unshift @_ => undef if scalar @_ <= 2 && (reftype($_[1]) || '') eq 'CODE';
+    unshift @_ => undef if scalar @_ <= 2 && ('CODE' eq ref($_[1]));
     goto &_create_type_constraint;
 }
 
+sub class_type ($;$) {
+    register_type_constraint(
+        create_class_type_constraint(
+            $_[0],
+            ( defined($_[1]) ? $_[1] : () ),
+        )
+    );
+}
+
+sub role_type ($;$) {
+    register_type_constraint(
+        create_role_type_constraint(
+            $_[0],
+            ( defined($_[1]) ? $_[1] : () ),
+        )
+    );
+}
+
 sub coerce ($@) {
     my ($type_name, @coercion_map) = @_;
     _install_type_coercions($type_name, \@coercion_map);
@@ -228,13 +327,32 @@ sub optimize_as (&) { +{ optimized => $_[0] } }
 
 sub enum ($;@) {
     my ($type_name, @values) = @_;
+    # NOTE:
+    # if only an array-ref is passed then
+    # you get an anon-enum
+    # - SL
+    if (ref $type_name eq 'ARRAY' && !@values) {
+        @values    = @$type_name;
+        $type_name = undef;
+    }
     (scalar @values >= 2)
         || confess "You must have at least two values to enumerate through";
     my %valid = map { $_ => 1 } @values;
-    _create_type_constraint(
-        $type_name,
-        'Str',
-        sub { $valid{$_} }
+
+    register_type_constraint(
+        create_enum_type_constraint(
+            $type_name,
+            \@values,
+        )
+    );
+}
+
+sub create_enum_type_constraint ($$) {
+    my ( $type_name, $values ) = @_;
+    
+    Moose::Meta::TypeConstraint::Enum->new(
+        name   => $type_name || '__ANON__',
+        values => $values,
     );
 }
 
@@ -245,7 +363,7 @@ sub enum ($;@) {
 sub _create_type_constraint ($$$;$$) {
     my $name   = shift;
     my $parent = shift;
-    my $check  = shift || sub { 1 };
+    my $check  = shift;
 
     my ($message, $optimized);
     for (@_) {
@@ -265,9 +383,15 @@ sub _create_type_constraint ($$$;$$) {
                  if defined $type;
     }
 
-    $parent = find_or_create_type_constraint($parent) if defined $parent;
+    my $class = "Moose::Meta::TypeConstraint";
 
-    my $constraint = Moose::Meta::TypeConstraint->new(
+    # FIXME should probably not be a special case
+    if ( defined $parent and $parent = find_or_parse_type_constraint($parent) ) {
+        $class = "Moose::Meta::TypeConstraint::Parameterizable" 
+            if $parent->isa("Moose::Meta::TypeConstraint::Parameterizable");
+    }
+
+    my $constraint = $class->new(
         name               => $name || '__ANON__',
         package_defined_in => $pkg_defined_in,
 
@@ -277,6 +401,21 @@ sub _create_type_constraint ($$$;$$) {
         ($optimized ? (optimized  => $optimized) : ()),
     );
 
+    # NOTE:
+    # if we have a type constraint union, and no
+    # type check, this means we are just aliasing
+    # the union constraint, which means we need to
+    # handle this differently.
+    # - SL
+    if (not(defined $check)
+        && $parent->isa('Moose::Meta::TypeConstraint::Union')
+        && $parent->has_coercion
+        ){
+        $constraint->coercion(Moose::Meta::TypeCoercion::Union->new(
+            type_constraint => $parent
+        ));
+    }
+
     $REGISTRY->add_type_constraint($constraint)
         if defined $name;
 
@@ -286,13 +425,18 @@ sub _create_type_constraint ($$$;$$) {
 sub _install_type_coercions ($$) {
     my ($type_name, $coercion_map) = @_;
     my $type = $REGISTRY->get_type_constraint($type_name);
-    (!$type->has_coercion)
-        || confess "The type coercion for '$type_name' has already been registered";
-    my $type_coercion = Moose::Meta::TypeCoercion->new(
-        type_coercion_map => $coercion_map,
-        type_constraint   => $type
-    );
-    $type->coercion($type_coercion);
+    (defined $type)
+        || confess "Cannot find type '$type_name', perhaps you forgot to load it.";
+    if ($type->has_coercion) {
+        $type->coercion->add_type_coercions(@$coercion_map);
+    }
+    else {
+        my $type_coercion = Moose::Meta::TypeCoercion->new(
+            type_coercion_map => $coercion_map,
+            type_constraint   => $type
+        );
+        $type->coercion($type_coercion);
+    }
 }
 
 ## --------------------------------------------------------
@@ -311,6 +455,8 @@ sub _install_type_coercions ($$) {
     my $valid_chars = qr{[\w:]};
     my $type_atom   = qr{ $valid_chars+ };
 
+    my $any;
+
     my $type                = qr{  $valid_chars+  (?: \[  (??{$any})  \] )? }x;
     my $type_capture_parts  = qr{ ($valid_chars+) (?: \[ ((??{$any})) \] )? }x;
     my $type_with_parameter = qr{  $valid_chars+      \[  (??{$any})  \]    }x;
@@ -318,22 +464,25 @@ sub _install_type_coercions ($$) {
     my $op_union = qr{ \s* \| \s* }x;
     my $union    = qr{ $type (?: $op_union $type )+ }x;
 
-    our $any = qr{ $type | $union }x;
+    $any = qr{ $type | $union }x;
 
     sub _parse_parameterized_type_constraint {
+        { no warnings 'void'; $any; } # force capture of interpolated lexical
         $_[0] =~ m{ $type_capture_parts }x;
         return ($1, $2);
     }
 
     sub _detect_parameterized_type_constraint {
+        { no warnings 'void'; $any; } # force capture of interpolated lexical
         $_[0] =~ m{ ^ $type_with_parameter $ }x;
     }
 
     sub _parse_type_constraint_union {
+        { no warnings 'void'; $any; } # force capture of interpolated lexical
         my $given = shift;
         my @rv;
         while ( $given =~ m{ \G (?: $op_union )? ($type) }gcx ) {
-                push @rv => $1;
+            push @rv => $1;
         }
         (pos($given) eq length($given))
             || confess "'$given' didn't parse (parse-pos="
@@ -345,6 +494,7 @@ sub _install_type_coercions ($$) {
     }
 
     sub _detect_type_constraint_union {
+        { no warnings 'void'; $any; } # force capture of interpolated lexical
         $_[0] =~ m{^ $type $op_union $type ( $op_union .* )? $}x;
     }
 }
@@ -366,88 +516,130 @@ subtype 'Bool'
 subtype 'Value'
     => as 'Defined'
     => where { !ref($_) }
-    => optimize_as { defined($_[0]) && !ref($_[0]) };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Value;
 
 subtype 'Ref'
     => as 'Defined'
     => where {  ref($_) }
-    => optimize_as { ref($_[0]) };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Ref;
 
 subtype 'Str'
     => as 'Value'
     => where { 1 }
-    => optimize_as { defined($_[0]) && !ref($_[0]) };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Str;
 
 subtype 'Num'
     => as 'Value'
     => where { Scalar::Util::looks_like_number($_) }
-    => optimize_as { !ref($_[0]) && Scalar::Util::looks_like_number($_[0]) };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Num;
 
 subtype 'Int'
     => as 'Num'
     => where { "$_" =~ /^-?[0-9]+$/ }
-    => optimize_as { defined($_[0]) && !ref($_[0]) && $_[0] =~ /^-?[0-9]+$/ };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Int;
 
-subtype 'ScalarRef' => as 'Ref' => where { ref($_) eq 'SCALAR' } => optimize_as { ref($_[0]) eq 'SCALAR' };
-subtype 'ArrayRef'  => as 'Ref' => where { ref($_) eq 'ARRAY'  } => optimize_as { ref($_[0]) eq 'ARRAY'  };
-subtype 'HashRef'   => as 'Ref' => where { ref($_) eq 'HASH'   } => optimize_as { ref($_[0]) eq 'HASH'   };
-subtype 'CodeRef'   => as 'Ref' => where { ref($_) eq 'CODE'   } => optimize_as { ref($_[0]) eq 'CODE'   };
-subtype 'RegexpRef' => as 'Ref' => where { ref($_) eq 'Regexp' } => optimize_as { ref($_[0]) eq 'Regexp' };
-subtype 'GlobRef'   => as 'Ref' => where { ref($_) eq 'GLOB'   } => optimize_as { ref($_[0]) eq 'GLOB'   };
+subtype 'ScalarRef' => as 'Ref' => where { ref($_) eq 'SCALAR' } => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::ScalarRef;
+subtype 'CodeRef'   => as 'Ref' => where { ref($_) eq 'CODE'   } => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::CodeRef;
+subtype 'RegexpRef' => as 'Ref' => where { ref($_) eq 'Regexp' } => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::RegexpRef;
+subtype 'GlobRef'   => as 'Ref' => where { ref($_) eq 'GLOB'   } => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::GlobRef;
 
 # NOTE:
 # scalar filehandles are GLOB refs,
 # but a GLOB ref is not always a filehandle
 subtype 'FileHandle'
     => as 'GlobRef'
-    => where { Scalar::Util::openhandle($_) }
-    => optimize_as { ref($_[0]) eq 'GLOB' && Scalar::Util::openhandle($_[0]) };
+    => where { Scalar::Util::openhandle($_) || ( blessed($_) && $_->isa("IO::Handle") ) }
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::FileHandle;
 
 # NOTE:
 # blessed(qr/.../) returns true,.. how odd
 subtype 'Object'
     => as 'Ref'
     => where { blessed($_) && blessed($_) ne 'Regexp' }
-    => optimize_as { blessed($_[0]) && blessed($_[0]) ne 'Regexp' };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Object;
 
 subtype 'Role'
     => as 'Object'
     => where { $_->can('does') }
-    => optimize_as { blessed($_[0]) && $_[0]->can('does') };
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::Role;
 
 my $_class_name_checker = sub {
-    return if ref($_[0]);
-    return unless defined($_[0]) && length($_[0]);
-
-    # walk the symbol table tree to avoid autovififying
-    # \*{${main::}{"Foo::"}} == \*main::Foo::
-
-    my $pack = \*::;
-    foreach my $part (split('::', $_[0])) {
-        return unless exists ${$$pack}{"${part}::"};
-        $pack = \*{${$$pack}{"${part}::"}};
-    }
-
-    # check for $VERSION or @ISA
-    return 1 if exists ${$$pack}{VERSION}
-             && defined *{${$$pack}{VERSION}}{SCALAR};
-    return 1 if exists ${$$pack}{ISA}
-             && defined *{${$$pack}{ISA}}{ARRAY};
-
-    # check for any method
-    foreach ( keys %{$$pack} ) {
-        next if substr($_, -2, 2) eq '::';
-        return 1 if defined *{${$$pack}{$_}}{CODE};
-    }
-
-    # fail
-    return;
 };
 
 subtype 'ClassName'
     => as 'Str'
-    => $_class_name_checker # where ...
-    => { optimize => $_class_name_checker };
+    => where { Class::MOP::is_class_loaded($_) }
+    => optimize_as \&Moose::Util::TypeConstraints::OptimizedConstraints::ClassName;
+
+## --------------------------------------------------------
+# parameterizable types ...
+
+$REGISTRY->add_type_constraint(
+    Moose::Meta::TypeConstraint::Parameterizable->new(
+        name                 => 'ArrayRef',
+        package_defined_in   => __PACKAGE__,
+        parent               => find_type_constraint('Ref'),
+        constraint           => sub { ref($_) eq 'ARRAY'  },
+        optimized            => \&Moose::Util::TypeConstraints::OptimizedConstraints::ArrayRef,
+        constraint_generator => sub {
+            my $type_parameter = shift;
+            my $check = $type_parameter->_compiled_type_constraint;
+            return sub {
+                foreach my $x (@$_) {
+                    ($check->($x)) || return
+                } 1;
+            }
+        }
+    )
+);
+
+$REGISTRY->add_type_constraint(
+    Moose::Meta::TypeConstraint::Parameterizable->new(
+        name                 => 'HashRef',
+        package_defined_in   => __PACKAGE__,
+        parent               => find_type_constraint('Ref'),
+        constraint           => sub { ref($_) eq 'HASH'  },
+        optimized            => \&Moose::Util::TypeConstraints::OptimizedConstraints::HashRef,
+        constraint_generator => sub {
+            my $type_parameter = shift;
+            my $check = $type_parameter->_compiled_type_constraint;
+            return sub {
+                foreach my $x (values %$_) {
+                    ($check->($x)) || return
+                } 1;
+            }
+        }
+    )
+);
+
+$REGISTRY->add_type_constraint(
+    Moose::Meta::TypeConstraint::Parameterizable->new(
+        name                 => 'Maybe',
+        package_defined_in   => __PACKAGE__,
+        parent               => find_type_constraint('Item'),
+        constraint           => sub { 1 },
+        constraint_generator => sub {
+            my $type_parameter = shift;
+            my $check = $type_parameter->_compiled_type_constraint;
+            return sub {
+                return 1 if not(defined($_)) || $check->($_);
+                return;
+            }
+        }
+    )
+);
+
+my @PARAMETERIZABLE_TYPES = map {
+    $REGISTRY->get_type_constraint($_)
+} qw[ArrayRef HashRef Maybe];
+
+sub get_all_parameterizable_types { @PARAMETERIZABLE_TYPES }
+sub add_parameterizable_type {
+    my $type = shift;
+    (blessed $type && $type->isa('Moose::Meta::TypeConstraint::Parameterizable'))
+        || confess "Type must be a Moose::Meta::TypeConstraint::Parameterizable not $type";
+    push @PARAMETERIZABLE_TYPES => $type;
+}
 
 ## --------------------------------------------------------
 # end of built-in types ...
@@ -501,11 +693,13 @@ and they are not used by Moose unless you tell it to. No type
 inference is performed, expression are not typed, etc. etc. etc.
 
 This is simply a means of creating small constraint functions which
-can be used to simplify your own type-checking code.
+can be used to simplify your own type-checking code, with the added 
+side benefit of making your intentions clearer through self-documentation.
 
 =head2 Slightly Less Important Caveat
 
-It is almost always a good idea to quote your type and subtype names.
+It is B<always> a good idea to quote your type and subtype names.
+
 This is to prevent perl from trying to execute the call as an indirect
 object call. This issue only seems to come up when you have a subtype
 the same name as a valid class, but when the issue does arise it tends
@@ -529,12 +723,13 @@ yet to have been created yet, is to simply do this:
 
 =head2 Default Type Constraints
 
-This module also provides a simple hierarchy for Perl 5 types, this
-could probably use some work, but it works for me at the moment.
+This module also provides a simple hierarchy for Perl 5 types, here is 
+that hierarchy represented visually.
 
   Any
   Item
       Bool
+      Maybe[`a]
       Undef
       Defined
           Value
@@ -544,8 +739,8 @@ could probably use some work, but it works for me at the moment.
                 ClassName
           Ref
               ScalarRef
-              ArrayRef
-              HashRef
+              ArrayRef[`a]
+              HashRef[`a]
               CodeRef
               RegexpRef
               GlobRef
@@ -553,23 +748,35 @@ could probably use some work, but it works for me at the moment.
               Object
                   Role
 
-Suggestions for improvement are welcome.
+B<NOTE:> Any type followed by a type parameter C<[`a]> can be
+parameterized, this means you can say:
 
-B<NOTE:> The C<Undef> type constraint does not work correctly
-in every occasion, please use it sparringly.
+  ArrayRef[Int]    # an array of intergers
+  HashRef[CodeRef] # a hash of str to CODE ref mappings
+  Maybe[Str]       # value may be a string, may be undefined
 
-B<NOTE:> The C<ClassName> type constraint is simply a subtype
-of string which responds true to C<isa('UNIVERSAL')>. This means
-that your class B<must> be loaded for this type constraint to
-pass. I know this is not ideal for all, but it is a saner
-restriction than most others.
+B<NOTE:> The C<Undef> type constraint for the most part works
+correctly now, but edge cases may still exist, please use it
+sparringly.
+
+B<NOTE:> The C<ClassName> type constraint does a complex package
+existence check. This means that your class B<must> be loaded for
+this type constraint to pass. I know this is not ideal for all,
+but it is a saner restriction than most others.
+
+=head2 Type Constraint Naming 
+
+Since the types created by this module are global, it is suggested 
+that you namespace your types just as you would namespace your 
+modules. So instead of creating a I<Color> type for your B<My::Graphics>
+module, you would call the type I<My::Graphics::Color> instead.
 
 =head2 Use with Other Constraint Modules
 
 This module should play fairly nicely with other constraint
 modules with only some slight tweaking. The C<where> clause
 in types is expected to be a C<CODE> reference which checks
-it's first argument and returns a bool. Since most constraint
+it's first argument and returns a boolean. Since most constraint
 modules work in a similar way, it should be simple to adapt
 them to work with Moose.
 
@@ -581,7 +788,8 @@ L<Declare::Constraints::Simple> to declare a completely new type.
           -keys   => HasLength,
           -values => IsArrayRef( IsObject ));
 
-For more examples see the F<t/204_example_w_DCS.t> test file.
+For more examples see the F<t/200_examples/204_example_w_DCS.t> 
+test file.
 
 Here is an example of using L<Test::Deep> and it's non-test
 related C<eq_deeply> function.
@@ -595,70 +803,11 @@ related C<eq_deeply> function.
               })))
         };
 
-For a complete example see the F<t/205_example_w_TestDeep.t>
-test file.
+For a complete example see the 
+F<t/200_examples/205_example_w_TestDeep.t> test file.
 
 =head1 FUNCTIONS
 
-=head2 Type Constraint Construction & Locating
-
-=over 4
-
-=item B<create_type_constraint_union ($pipe_seperated_types | @type_constraint_names)>
-
-Given string with C<$pipe_seperated_types> or a list of C<@type_constraint_names>,
-this will return a L<Moose::Meta::TypeConstraint::Union> instance.
-
-=item B<create_parameterized_type_constraint ($type_name)>
-
-Given a C<$type_name> in the form of:
-
-  BaseType[ContainerType]
-
-this will extract the base type and container type and build an instance of
-L<Moose::Meta::TypeConstraint::Parameterized> for it.
-
-=item B<find_or_create_type_constraint ($type_name, ?$options_for_anon_type)>
-
-This will attempt to find or create a type constraint given the a C<$type_name>.
-If it cannot find it in the registry, it will see if it should be a union or
-container type an create one if appropriate, and lastly if nothing can be
-found or created that way, it will create an anon-type using the
-C<$options_for_anon_type> HASH ref to populate it. If the C<$options_for_anon_type>
-is not specified (it is C<undef>), then it will not create anything and simply
-return.
-
-=item B<find_type_constraint ($type_name)>
-
-This function can be used to locate a specific type constraint
-meta-object, of the class L<Moose::Meta::TypeConstraint> or a
-derivative. What you do with it from there is up to you :)
-
-=item B<get_type_constraint_registry>
-
-Fetch the L<Moose::Meta::TypeConstraint::Registry> object which
-keeps track of all type constraints.
-
-=item B<list_all_type_constraints>
-
-This will return a list of type constraint names, you can then
-fetch them using C<find_type_constraint ($type_name)> if you
-want to.
-
-=item B<list_all_builtin_type_constraints>
-
-This will return a list of builtin type constraints, meaning,
-those which are defined in this module. See the section
-labeled L<Default Type Constraints> for a complete list.
-
-=item B<export_type_constraints_as_functions>
-
-This will export all the current type constraints as functions
-into the caller's namespace. Right now, this is mostly used for
-testing, but it might prove useful to others.
-
-=back
-
 =head2 Type Constraint Constructors
 
 The following functions are used to create type constraints.
@@ -683,6 +832,16 @@ This creates an unnamed subtype and will return the type
 constraint meta-object, which will be an instance of
 L<Moose::Meta::TypeConstraint>.
 
+=item B<class_type ($class, ?$options)>
+
+Creates a type constraint with the name C<$class> and the metaclass
+L<Moose::Meta::TypeConstraint::Class>.
+
+=item B<role_type ($role, ?$options)>
+
+Creates a type constraint with the name C<$role> and the metaclass
+L<Moose::Meta::TypeConstraint::Role>.
+
 =item B<enum ($name, @values)>
 
 This will create a basic subtype for a given set of strings.
@@ -693,6 +852,17 @@ See the L<SYNOPSIS> for a simple example.
 B<NOTE:> This is not a true proper enum type, it is simple
 a convient constraint builder.
 
+=item B<enum (\@values)>
+
+If passed an ARRAY reference instead of the C<$name>, C<@values> pair,
+this will create an unnamed enum. This can then be used in an attribute
+definition like so:
+
+  has 'sort_order' => (
+      is  => 'ro',
+      isa => enum([qw[ ascending descending ]]),
+  );
+
 =item B<as>
 
 This is just sugar for the type constraint construction syntax.
@@ -741,6 +911,106 @@ This is just sugar for the type coercion construction syntax.
 
 =back
 
+=head2 Type Constraint Construction & Locating
+
+=over 4
+
+=item B<create_type_constraint_union ($pipe_seperated_types | @type_constraint_names)>
+
+Given string with C<$pipe_seperated_types> or a list of C<@type_constraint_names>,
+this will return a L<Moose::Meta::TypeConstraint::Union> instance.
+
+=item B<create_parameterized_type_constraint ($type_name)>
+
+Given a C<$type_name> in the form of:
+
+  BaseType[ContainerType]
+
+this will extract the base type and container type and build an instance of
+L<Moose::Meta::TypeConstraint::Parameterized> for it.
+
+=item B<create_class_type_constraint ($class, ?$options)>
+
+Given a class name it will create a new L<Moose::Meta::TypeConstraint::Class>
+object for that class name.
+
+=item B<create_role_type_constraint ($role, ?$options)>
+
+Given a role name it will create a new L<Moose::Meta::TypeConstraint::Role>
+object for that role name.
+
+=item B<create_enum_type_constraint ($name, $values)>
+
+=item B<find_or_parse_type_constraint ($type_name)>
+
+This will attempt to find or create a type constraint given the a C<$type_name>.
+If it cannot find it in the registry, it will see if it should be a union or
+container type an create one if appropriate
+
+=item B<find_or_create_type_constraint ($type_name, ?$options_for_anon_type)>
+
+This function will first call C<find_or_parse_type_constraint> with the type name.
+
+If no type is found or created, but C<$options_for_anon_type> are provided, it
+will create the corresponding type.
+
+This was used by the C<does> and C<isa> parameters to L<Moose::Meta::Attribute>
+and are now superseded by C<find_or_create_isa_type_constraint> and
+C<find_or_create_does_type_constraint>.
+
+=item B<find_or_create_isa_type_constraint ($type_name)>
+
+=item B<find_or_create_does_type_constraint ($type_name)>
+
+Attempts to parse the type name using L<find_or_parse_type_constraint> and if
+no appropriate constraint is found will create a new anonymous one.
+
+The C<isa> variant will use C<create_class_type_constraint> and the C<does>
+variant will use C<create_role_type_constraint>.
+
+=item B<find_type_constraint ($type_name)>
+
+This function can be used to locate a specific type constraint
+meta-object, of the class L<Moose::Meta::TypeConstraint> or a
+derivative. What you do with it from there is up to you :)
+
+=item B<register_type_constraint ($type_object)>
+
+This function will register a named type constraint with the type registry.
+
+=item B<get_type_constraint_registry>
+
+Fetch the L<Moose::Meta::TypeConstraint::Registry> object which
+keeps track of all type constraints.
+
+=item B<list_all_type_constraints>
+
+This will return a list of type constraint names, you can then
+fetch them using C<find_type_constraint ($type_name)> if you
+want to.
+
+=item B<list_all_builtin_type_constraints>
+
+This will return a list of builtin type constraints, meaning,
+those which are defined in this module. See the section
+labeled L<Default Type Constraints> for a complete list.
+
+=item B<export_type_constraints_as_functions>
+
+This will export all the current type constraints as functions
+into the caller's namespace. Right now, this is mostly used for
+testing, but it might prove useful to others.
+
+=item B<get_all_parameterizable_types>
+
+This returns all the parameterizable types that have been registered.
+
+=item B<add_parameterizable_type ($type)>
+
+Adds C<$type> to the list of parameterizable types
+
+=back
+
 =head2 Namespace Management
 
 =over 4
@@ -764,7 +1034,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006, 2007 by Infinity Interactive, Inc.
+Copyright 2006-2008 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
